import unittest
import os
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.structure.files import Files
from mkdocs_asyncapi_tag.mkdocs_asyncapi_plugin import AsyncAPIPlugin

class TestAsyncAPIPlugin(unittest.TestCase):

    def setUp(self):
        self.plugin = AsyncAPIPlugin()
        self.config = MkDocsConfig()
        self.config['docs_dir'] = 'docs'
        self.config['site_dir'] = 'site'
        self.config['use_directory_urls'] = False

        # Create a mock schema.json file
        os.makedirs(self.config['docs_dir'], exist_ok=True)
        with open(os.path.join(self.config['docs_dir'], 'schema.json'), 'w') as f:
            f.write('{}')

    def tearDown(self):
        # Clean up the mock file after tests
        os.remove(os.path.join(self.config['docs_dir'], 'schema.json'))

    def test_on_files(self):
        files = Files([])
        self.plugin.config['asyncapi_file'] = 'schema.json'
        result = self.plugin.on_files(files, self.config)
        
        # Check if the file was added to the Files object
        file_paths = [file.src_path for file in result]
        self.assertIn('schema.json', file_paths)

    def test_on_page_content(self):
        html = '<asyncapi-tag src="schema.json"></asyncapi-tag>'
        page = None
        files = None
        result = self.plugin.on_page_content(html, page, self.config, files)
        self.assertIn('schema.json', result)

if __name__ == '__main__':
    unittest.main()



from mkdocs.plugins import BasePlugin
from mkdocs.config import config_options
from mkdocs.structure.files import File
import os
import re
import json

class AsyncAPIPlugin(BasePlugin):

    config_scheme = (
        ('asyncapi_file', config_options.Type(str, default='schema.json')),
    )

    def on_files(self, files, config):
        """
        Add the AsyncAPI file to the MkDocs files structure.
        """
        asyncapi_file_path = self.config['asyncapi_file']

        file_path = os.path.join(config['docs_dir'], asyncapi_file_path)
        if os.path.exists(file_path):
            # Create a File object for the AsyncAPI file
            file = File(
                path=asyncapi_file_path,
                src_dir=config['docs_dir'],
                dest_dir=config['site_dir'],
                use_directory_urls=config['use_directory_urls']
            )
            files.append(file)
            print(f"Added AsyncAPI schema: {file_path}")
        else:
            print(f"Warning: The specified AsyncAPI file '{file_path}' does not exist.")
        
        return files

    def on_page_content(self, html, page, config, files):
        """
        Inject AsyncAPI viewer into the relevant HTML file after it is processed.
        """

        match = re.search(r'<asyncapi-tag\s+([^>]+)>', html)
        if match:
            # Extract the JSON string from the custom tag
            tag_content = match.group(1)
            attributes = dict(re.findall(r'(\w+)\s*=\s*"([^"]+)"', tag_content))
            try:
                # configuration for show property
                schema_path = os.path.join(config['docs_dir'],attributes.get('src', 'schema.json'))
                sidebar = attributes.get('sidebar', 'true').lower() == 'true'
                info = attributes.get('info', 'true').lower() == 'true'
                servers = attributes.get('servers', 'true').lower() == 'true'
                operations = attributes.get('operations', 'true').lower() == 'true'
                messages = attributes.get('messages', 'true').lower() == 'true'
                schemas = attributes.get('schemas', 'true').lower() == 'true'
                errors = attributes.get('errors', 'true').lower() == 'true'

                #configuration for expand property
                messageExamples = attributes.get('messageExamples', 'true').lower() == 'true'

                #configuration for sidebar property
                try:
                    showServers = attributes.get('showServers', 'byDefault') == 'byDefault'
                except:
                    print("Error: Invalid showServers attribute value in the asyncapi tag. Supported values include: byDefault, bySpecTags, byServerTags")

                try:
                    showOperations = attributes.get('showOperations', 'byDefault') == 'byDefault'
                except:
                    print("Error: Invalid showServers attribute value in the asyncapi tag. Supported values include: byDefault, bySpecTags, byOperationsTags")

                # configuration for parser options, publish label and subscribe label
                try:
                    parserOptions = attributes.get('parserOptions', 'null') == 'null'
                except:
                    print("Error: Invalid parserOptions attribute value in the asyncapi tag.")

                publishLabel = attributes.get('publishLabel', 'PUB') == 'PUB'
                subscribeLabel = attributes.get('subscribeLabel', 'SUB') == 'SUB'

                viewer_config = {"expand":{"messageExamples":messageExamples},"sidebar":{"showServers":showServers,"showOperations":showOperations},"parserOptions":parserOptions,"publishLabel":publishLabel,"subscribeLabel":subscribeLabel,"show": {"sidebar": sidebar, "info":info, "servers": servers, "operations":operations, "messages": messages, "schemas":schemas, "errors":errors}}
            except json.JSONDecodeError:
                print("Error: Invalid attribute in the asyncapi tag")
                return html
            
            asyncapi_viewer = f'''
            <div id="asyncapi"></div>
            <script src="https://unpkg.com/js-yaml@4.0.0/dist/js-yaml.min.js"></script>
            <script src="https://unpkg.com/@asyncapi/react-component@latest/browser/standalone/index.js"></script>
            <script>

                console.log('Initializing AsyncAPI Viewer...');
                
                const schemaPath = '{schema_path}'; 
                console.log('Fetching schema from:', schemaPath);

                // Fetch the schema file to ensure it is accessible
                fetch(schemaPath).then(response => {{
                    if (!response.ok) {{
                        throw new Error(`Failed to fetch schema. Status: ${{response.status}}`);
                    }}
                    return response.text();
                }}).then(schemaText => {{
                    
                    let schemaDoc;
                    // Check if the schema is in JSON or YAML format based on the file extension or content
                    if (schemaPath.endWith('.yaml') || (schemaPath.endWith('.yml') {{
                        // Parse YAML to JSON
                        schemaDoc = jsyaml.load(schemaText);
                        console.log('YAML schema parsed:', schemaDoc);
                    }} else {{
                        // Parse JSON
                        schemaDoc = JSON.parse(schemaText);
                        console.log('JSON schema parsed:', schemaDoc);
                    }}

                    console.log('Schema fetched successfully:', schemaDoc);
                    AsyncApiStandalone.render({{
                        schema: schemaDoc,
                        config: {json.dumps(viewer_config)}
                    }}, document.getElementById('asyncapi'));

                    console.log('AsyncAPI Viewer Initialized');

                }}).catch(error => {{
                    console.error('Error fetching or rendering AsyncAPI schema:', error);
                }});

            </script>
            '''
            # Inject the AsyncAPI viewer just before closing body tag
            html += asyncapi_viewer
        return html




from setuptools import setup, find_packages

setup(
    name='mkdocs-asyncapi-tag-plugin',
    version='0.9.0',
    description='MkDocs plugin to embed AsyncAPI HTML viewer in your markdown file.',
    author='Weesho Lapara',
    author_email='support@weesholapara.com',
    url='https://github.com/Weesho-Lapara/mkdocs-asyncapi-tag-plugin',
    packages=find_packages(),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    keywords=["mkdocs asyncapi", "asyncapi render markdown"],
    install_requires=[
        'mkdocs>=1.0.4',
        'beautifulsoup4>=4.6.0',
    ],
    entry_points={
        'mkdocs.plugins': [
            'asyncapi-tag = mkdocs_asyncapi_tag.mkdocs_asyncapi_plugin:AsyncAPIPlugin',
        ]
    },
    include_package_data=True
)
